﻿using Refund_CSharp.Soap;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Refund_CSharp.BusinessLogic
{
    public class SOPGRefundMerchantClient
    {
        private Dictionary<String, String> transaction = new Dictionary<String, String>();
        private SOAPError error = new SOAPError();
        private PSCSOAPClient client;
        private bool productive;
        private Dictionary<String, String> refundResult = new Dictionary<String, String>();

        private const String TEST_URL = "https://soatest.paysafecard.com/psc/services/PscService?wsdl";
        private const String PRODUCTIVE_URL = "https://soa.paysafecard.com/psc/services/PscService?wsdl";

        public SOPGRefundMerchantClient(String username, String password, bool productive)
        {
            transaction.Add("username", username);
            transaction.Add("password", password);
            this.productive = productive;
        }

        public void SetTransaction(String amount, String currency, String mtid, String rtid, String utcOffset)
        {

            if (rtid != null && !rtid.Equals(""))
                transaction.Add("rtid", rtid);
            else
                transaction.Add("rtid", generateRtid());

            transaction.Add("mtid", mtid);

            amount = amount.Replace(",", ".");
            double amountDouble = Math.Round(double.Parse(amount), 2);
            transaction.Add("amount", amountDouble.ToString());

            transaction.Add("currency", currency.ToUpper());

            if (utcOffset == null || utcOffset.Equals(""))
                utcOffset = "+01:00";
            transaction.Add("utcOffset", utcOffset);
        }

        public void SetCustomer(String merchantClientId, String customerId, String customerIdType)
        {
            if (customerIdType == null || customerIdType.Equals(""))
                customerIdType = "EMAIL";
            transaction.Add("customerIdType", customerIdType);

            transaction.Add("customerId", customerId);
            transaction.Add("merchantClientId", merchantClientId);
        }

        public Dictionary<String, String> Refund(bool validationOnly)
        {
            if (validationOnly)
                transaction.Add("validationOnly", "true");
            else
                transaction.Add("validationOnly", "false");

            if (client == null)
                connect();

            client.CreateEnvelope("refund", transaction);

            if (productive)
                client.doCall(PRODUCTIVE_URL);
            else
                client.doCall(TEST_URL);

            RefundResult();

            return refundResult;
        }

        private void RefundResult()
        {
            refundResult.Add("status", client.GetElement("errorCode").Equals("0")
                    && client.GetElement("resultCode").Equals("0") ? "OK" : "ERROR");

            if (client.GetElement("errorCode").Equals("0") && client.GetElement("resultCode").Equals("0"))
            {
                refundResult.Add("errorMsg", "");
            }
            else
            {
                setError(client.GetElement("errorCode"), client.GetElement("errorCodeDescription"));
                refundResult.Add("errorMsg", error.ErrorMessage);
            }

            refundResult.Add("rtid", client.GetElement("rtid"));
            refundResult.Add("mtid", client.GetElement("mtid"));
            refundResult.Add("validationOnly", client.GetElement("validationOnly"));

            refundResult.Add("requestedCurrency", client.GetElement("requestedCurrency"));
            refundResult.Add("requestedAmount", client.GetElement("requestedAmount"));
            refundResult.Add("refundedInCurrency", client.GetElement("refundedInCurrency"));
            refundResult.Add("refundedAmount", client.GetElement("refundedAmount"));

            refundResult.Add("resultCode", client.GetElement("resultCode"));
            refundResult.Add("errorCode", client.GetElement("errorCode"));
            refundResult.Add("errorCodeDescription", client.GetElement("errorCodeDescription"));
        }

        private void connect()
        {
            client = new PSCSOAPClient();
        }

        private String generateRtid()
        {
            DateTime today = DateTime.Now;
            string format = "ddMMyyyy";
            Random rnd = new Random();
            return "refundTest_" + today.ToString(format) + "_" + rnd.Next(1, 1000);
        }

        public Dictionary<String, Dictionary<String, String>> getDetails()
        {
            Dictionary<String, Dictionary<String, String>> details = new Dictionary<String, Dictionary<String, String>>();
            details.Add("transaction", transaction);
            details.Add("refundResult", refundResult);

            return details;
        }

        private void setError(String errorCode, String errorCodeDesc)
        {
            if (errorCodeDesc == null)
                errorCodeDesc = "";

            switch (errorCode)
            {
                case "3184":
                    error.ErrorMessage = "Original payment transaction does not exist.";
                    break;
                case "3162":
                    error.ErrorMessage = "E-mail address is not registered with mypaysafecard.";
                    break;
                case "3182":
                    error.ErrorMessage = "Merchantclient-ID is a mandatory parameter.";
                    break;
                case "3181":
                    error.ErrorMessage = "Merchantlcient-ID is not matching with original transaction";
                    break;
                default:
                    error.ErrorMessage = "A technical error occurred. ErrorCode: " + errorCode + " - " + errorCodeDesc
                            + ". Please contact your support department.";
                    break;
            }
            error.ErrorCode = errorCode;
        }
    }
}
