﻿using Refund_CSharp.Util;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading.Tasks;
using System.Xml;

namespace Refund_CSharp.Soap
{
    public class PSCSOAPClient
    {
        private StringBuilder soapRequest = new StringBuilder("");
        private XmlDocument soapResponse;

        public void doCall(string url)
        {
            ServicePointManager.Expect100Continue = true;

            WebRequest request = WebRequest.Create(url);
            HttpWebRequest httpRequest = (HttpWebRequest)request;
            httpRequest.Method = "POST";
            httpRequest.ContentType = "text/xml; charset=utf-8";
            httpRequest.Accept = "text/xml";
            httpRequest.KeepAlive = false;
            httpRequest.UserAgent = "Paysafecard refund test client";
            httpRequest.ReadWriteTimeout = 10000;
            httpRequest.ProtocolVersion = HttpVersion.Version10;
            httpRequest.Credentials = CredentialCache.DefaultCredentials;

            using (Stream stream = httpRequest.GetRequestStream())
            {
                using (StreamWriter streamWriter = new StreamWriter(stream, Encoding.ASCII))
                {
                    streamWriter.Write(soapRequest.ToString());
                    streamWriter.Close();
                }
            }

            String response = "";
            try
            {
                HttpWebResponse httpResponse = httpRequest.GetResponse() as HttpWebResponse;
                StreamReader streamReader = new StreamReader(httpResponse.GetResponseStream());
                response = streamReader.ReadToEnd();
            }
            catch (WebException ex)
            {
                ErrorHandler.HandleError(ex);
            }

            soapResponse = new XmlDocument();
            soapResponse.LoadXml(response);
        }

        public void CreateEnvelope(String parentElement, Dictionary<string, string> entries)
        {
            soapRequest.Clear();

            soapRequest.Append("<SOAP-ENV:Envelope xmlns:SOAP-ENV=\"http://schemas.xmlsoap.org/soap/envelope/\" xmlns:urn=\"urn:pscservice\">");
            soapRequest.Append("<SOAP-ENV:Header/>");
            soapRequest.Append("<SOAP-ENV:Body>");
            soapRequest.Append("<urn:" + parentElement + ">");

            foreach (KeyValuePair<string, string> keyValue in entries)
            {
                soapRequest.Append("<urn:" + keyValue.Key + ">" + keyValue.Value + "</urn:" + keyValue.Key + ">");
            }

            soapRequest.Append("</urn:" + parentElement + ">");
            soapRequest.Append("</SOAP-ENV:Body>");
            soapRequest.Append("</SOAP-ENV:Envelope>");
        }

        public String GetElement(String name)
        {
            XmlNodeList nodeList = soapResponse.GetElementsByTagName("ns1:" + name);
            if (nodeList.Count > 1)
                throw new Exception("Multiple elements with this tagname where found!");
            else if (nodeList.Count == 1)
                return nodeList.Item(0).InnerText;
            else
                return "";
        }
    }
}
